package com.huiju.estateagents.center.taUser.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.center.sysUser.entity.SysToken;
import com.huiju.estateagents.center.sysUser.entity.SysUser;
import com.huiju.estateagents.center.sysUser.service.ISysTokenService;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.service.ITaUserService;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.JWTUtils;
import com.huiju.estateagents.common.MD5Utils;
import com.huiju.estateagents.entity.TaPerson;
import com.huiju.estateagents.entity.TaRole;
import com.huiju.estateagents.service.ITaRoleService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import sun.dc.pr.PRError;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 用户表  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-09-11
 */
@RestController
@RequestMapping("/api")
public class TaUserController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaUserController.class);

    @Autowired
    public ITaUserService iTaUserService;
    
    @Autowired
    private ISysTokenService sysTokenService;

    @Autowired
    private ITaRoleService iTaRoleService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/center/taUser",method= RequestMethod.GET)
    public ResponseBean taUserList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage pg = new Page<>(pageNum, pageSize);
            IPage<Map<String,Object>> result = iTaUserService.getPageList(pg);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taUser 实体对象
     * @return
     */
    @RequestMapping(value="/center/taUser",method= RequestMethod.POST)
    public ResponseBean taUserAdd(@RequestBody TaUser taUser){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaUserService.newUserAndOrg(taUser));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @PutMapping(value = "/center/turn/taUser/{id}/{type}")
    public ResponseBean turnUser(@PathVariable String type, @PathVariable Integer id) {
        try {
            iTaUserService.turnUserOnOrNot(id, "on".equals(type));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseBean.error(e.getMessage(), ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success("ok");
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taUser/{id}", method= RequestMethod.DELETE)
    public ResponseBean taUserDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaUserService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taUser 实体对象
     * @return
     */
    @RequestMapping(value="/center/taUser/{id}",method= RequestMethod.PUT)
    public ResponseBean taUserUpdate(@PathVariable Integer id,
                                        @RequestBody TaUser taUser){
        ResponseBean responseBean = new ResponseBean();
        try {
            iTaUserService.updateInfoWithOrg(taUser);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
    /**
     * 获取用户信息
     */
    @RequestMapping(value="/center/taUser/{id}",method= RequestMethod.GET)
    public ResponseBean taUserGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaUserService.getInfoWithOrg(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }

        return responseBean;
    }

    /**
     * 重置密码
     * @param plat
     * @param id
     * @return
     */
    @PutMapping(value = "/{plat}/password/taUser/{id}")
    public ResponseBean resetPassword(@PathVariable String plat, @PathVariable Integer id, HttpServletRequest request) {
        if (!"center".equals(plat) && !"admin".equals(plat)) {
            return ResponseBean.error("您没有权限进行此操作", ResponseBean.ERROR_AUTH_EXPIRED);
        }

        String newPass = plat.equals("center") ? "abc@123" : "123456";
        try {
            iTaUserService.resetPassword(id, newPass);
            return ResponseBean.success("新密码为: " + newPass);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseBean.error(e.getMessage(), ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 获取当前用户信息
     * @param request
     * @return
     */
    @RequestMapping(value="/admin/taUser/current",method= RequestMethod.GET)
    public ResponseBean taUserGetCurrent(HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        Map map = JWTUtils.getUserIdAndOrgId(request);
        try {
            TaUser taUser = iTaUserService.getById(map.get("userId").toString());
            List<TaRole> roles = iTaRoleService.getRolesByUser(taUser.getUserId());
            taUser.setRoles(roles);
            taUser.setLoginPassword("");
            responseBean.addSuccess(taUser);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
    
    /**
     * 用户手机号登录
     * @param paramStr
     * @return
     */
    @PostMapping("/admin/taUser/signin")
    public ResponseBean signin(@RequestBody String paramStr) {
        JSONObject params = JSONObject.parseObject(paramStr);
        if (params == null) {
            return ResponseBean.error("非法参数", ResponseBean.ERROR_MISSING_PARAMS);
        }
    
        String userName = params.getString("loginName");
        String userPassword = params.getString("loginPassword");
    
        //验证用户名密码是否正确
        QueryWrapper<TaUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.eq("login_name",userName);
        sysUserQueryWrapper.eq("login_password", MD5Utils.md5(userPassword));
        TaUser taUser = iTaUserService.getOne(sysUserQueryWrapper);
        if (taUser == null) {
            return ResponseBean.error("用户名或密码错误", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
    
        if (taUser.getStatus() != CommConstant.STATUS_NORMAL) {
            return ResponseBean.error("用户状态异常", ResponseBean.ERROR_UNAVAILABLE);
        }

        List<TaRole> roles = iTaRoleService.getRolesByUser(taUser.getUserId());
        taUser.setRoles(roles);
        taUser.setLoginPassword("");
        Map map = new HashMap();
        map.put("userId",taUser.getUserId().toString());
        map.put("orgId",taUser.getOrgId().toString());
        String token = JWTUtils.newTokenByMap(map);
    
        //保存token
        saveToken(token);
    
        Map<String, Object> result = new HashMap<>();
        result.put("token", token);
        result.put("user", taUser);
    
        return ResponseBean.success(result);
    }
    
    
    private void saveToken(String token) {
        SysToken sysToken = new SysToken();
        sysToken.setToken(token);
        sysToken.setExpDate(LocalDateTime.now());
        sysTokenService.save(sysToken);
    }
}
